﻿// JavaScript source code

var CrmApp = angular.module('crmApp', ['wj']);

// Data Service to make an WebAPI call to CRM
CrmApp.service('DataService', function ($q) {
    this.retrieveRecords = function () {
        return $q(function (resolve, reject) {
            var clientURL = Xrm.Page.context.getClientUrl();
            var req = new XMLHttpRequest()
            var query = "/api/data/v8.0/accounts?$select=name,address1_country,address1_composite,address1_city,address1_postalcode,donotphone,paymenttermscode&$top=100";
            req.open("GET", encodeURI(clientURL + query), true);
            req.setRequestHeader("Accept", "application/json");
            req.setRequestHeader("Content-Type", "application/json;charset=utf-8");
            req.setRequestHeader("OData-MaxVersion", "4.0");
            req.setRequestHeader("OData-Version", "4.0");
            req.onreadystatechange = function () {
                if (this.readyState == 4) {
                    req.onreadystatechange = null;
                    if (this.status == 200) {
                        var data = JSON.parse(this.response);
                        if (data && data.value) {
                            resolve(data.value);
                        }
                    }
                    else {
                        var error = JSON.parse(this.response).error;
                        reject(error);
                    }
                }
            };
            req.send(null);
        });
    }

    // Web Api call to Update Accounts
    this.updateRecord = function (accountId, record) {
        return $q(function (resolve, reject) {
            var clientURL = Xrm.Page.context.getClientUrl();
            var req = new XMLHttpRequest()
            req.open("PATCH", clientURL + "/api/data/v8.0/accounts(" + accountId + ")", true);
            req.setRequestHeader("Accept", "application/json");
            req.setRequestHeader("Content-Type", "application/json;charset=utf-8");
            req.setRequestHeader("OData-MaxVersion", "4.0");
            req.setRequestHeader("OData-Version", "4.0");
            req.onreadystatechange = function () {
                if (this.readyState == 4) {
                    req.onreadystatechange = null;
                    if (this.status == 204) {
                        resolve("ok");
                    }
                    else {
                        var error = JSON.parse(this.response).error;
                        reject(error);
                    }
                }
            };
            req.send(JSON.stringify(record));
        });
    }
});

/// create a controller for the page, and inject our DynamicsCRMService into it
CrmApp.controller('AppController', function ($scope, DataService, $q, $window, $timeout) {
    // Get Data for Accounts

    var rawData = [];

    //Get Data for Account Records
    $scope.GetData = function () {
        var promise = DataService.retrieveRecords();
        promise.then(function (response) {
            rawData = response;
            $scope.data = new wijmo.collections.CollectionView(response);
            $scope.data.trackChanges = true;

            $scope.AnalyzeAccounts();
        }, function (reason) {
            alert(reason)
        });
    }

    // Create Data for Country Analysis and Show in Pie Chart
    $scope.AnalyzeAccounts = function () {
        var countriesofAccount = [];
        var unsortedCountriesofAccount = rawData.reduce(function (sums, entry) {
            sums[entry.address1_country] = (sums[entry.address1_country] || 0) + 1;

            return sums;
        }, {});

        angular.forEach(unsortedCountriesofAccount, function (value, key) {
            var countriesData = {
                name: key,
                value: value
            };
            countriesofAccount.push(countriesData);
        })
        $scope.countriesofAccountCollection = new wijmo.collections.CollectionView(countriesofAccount);
    }

    // Udpate Account Edits to CRM
    $scope.UpdateRecords = function () {
        var defer = $q.defer();
        var promises = [];

        var edittedItems = $scope.data.itemsEdited;

        angular.forEach(edittedItems, function (item) {
            var account = {
                name: item.name,
                address1_line1: item.address1_line1,
                address1_country: item.address1_country,
                address1_city: item.address1_city,
                address1_postalcode: item.address1_postalcode,
                donotphone: item.donotphone,
                paymenttermscode: item.paymenttermscode
            };

            promises.push(DataService.updateRecord(item.accountid, account));
        })

        $q.all(promises).then($scope.GetData());
    }

    $scope.GetData();
});